@@ -33,7 +33,7 @@ GNOME_CLASS_BOILERPLATE (EelBackgroundStyle, eel_background_style,
 			 GtkStyle, GTK_TYPE_STYLE)
 
 struct EelBackgroundStyleDetails {
-	int dummy;
+	EelBackground *background;
 };
 
 static void
@@ -49,27 +49,15 @@ eel_background_style_finalize (GObject *object)
 
 	style = EEL_BACKGROUND_STYLE (object);
 
+	if (style->details->background) {
+		g_object_unref (style->details->background);
+	}
+	
 	g_free (style->details);
 
 	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
-static void
-eel_gdk_window_update_sizes (GdkWindow *window, int *width, int *height)
-{
-	g_return_if_fail (window != NULL);
-	g_return_if_fail (width != NULL);
-	g_return_if_fail (height != NULL);
-
-	if (*width == -1 && *height == -1) {
-		gdk_drawable_get_size (window, width, height);
-	} else if (*width == -1) {
-		gdk_drawable_get_size (window, width, NULL);
-	} else if (*height == -1) {
-		gdk_drawable_get_size (window, NULL, height);
-	}
-}
-
 static void
 eel_background_style_draw_flat_box (GtkStyle *style,
 				    GdkWindow *window,
@@ -86,7 +74,14 @@ eel_background_style_draw_flat_box (GtkStyle *style,
 	gboolean call_parent;
 	EelBackground *background;
 	GdkGC *gc;
-
+	GdkGCValues gc_values;
+	EelBackgroundStyleDetails *details;
+	GdkGCValuesMask value_mask;
+	GdkColor color;
+	GdkPixmap *pixmap;
+	gboolean changes_with_size;
+	int window_width, window_height;
+	
 	call_parent = TRUE;
 
 	background = NULL;
@@ -104,36 +99,68 @@ eel_background_style_draw_flat_box (GtkStyle *style,
 		return;
 	}
 
-    	gc = style->bg_gc[state_type];
 
-	if (area)
+	details = EEL_BACKGROUND_STYLE (style)->details;
+
+	gdk_drawable_get_size (window, &window_width, &window_height);
+	
+	pixmap = eel_background_get_pixmap_and_color (background, window,
+						      window_width, window_height,
+						      &color,
+						      &changes_with_size);
+
+	if (!changes_with_size) {
+		/* The background was already drawn by X, since we set
+		 * the GdkWindow background/back_pixmap.
+		 * No need to draw it again. */
+		return;
+	}
+	
+	if (pixmap != NULL) {
+		gc_values.tile = pixmap;
+		gc_values.ts_x_origin = 0;
+		gc_values.ts_y_origin = 0;
+		gc_values.fill = GDK_TILED;
+		value_mask = GDK_GC_FILL | GDK_GC_TILE |
+			GDK_GC_TS_X_ORIGIN | GDK_GC_TS_Y_ORIGIN;
+	} else {
+		gdk_rgb_find_color (gtk_widget_get_colormap (widget), &color);
+		gc_values.foreground = color;
+		gc_values.fill = GDK_SOLID;
+		value_mask = GDK_GC_FILL | GDK_GC_FOREGROUND;
+	}
+		
+    	gc = gdk_gc_new_with_values (window, &gc_values, value_mask);
+	
+	if (area != NULL) {
 		gdk_gc_set_clip_rectangle (gc, area);
+	}
 
-	eel_gdk_window_update_sizes (window, &width, &height);	
 
-	if (area)
-		eel_background_draw_to_drawable (background, window, gc,
-						 area->x, area->y,
-						 area->width, area->height,
-						 widget->allocation.width, widget->allocation.height);
-	else
-		eel_background_draw_to_drawable (background, window, gc,
-						 0, 0,
-						 width, height,
-						 widget->allocation.width, widget->allocation.height);
+	if (width == -1) {
+		width = window_width;
+	}
+	
+	if (height == -1) {
+		height = window_height;
+	}
 
-	if (area)
-		gdk_gc_set_clip_rectangle (gc, NULL);
+	gdk_draw_rectangle (window, gc, TRUE,
+			    x, y,
+			    width, height);
+
+	g_object_unref (gc);
+	
+	if (pixmap != NULL) {
+		g_object_unref (pixmap);
+	}
 }
 
-
 EelBackgroundStyle *
 eel_background_style_new (GtkStyle *base_style,
 			  EelBackground *background)
 {
 	EelBackgroundStyle *style;
-	char *background_color, *start_color_spec;
-	GdkColor color;
 	
 	g_return_val_if_fail (GTK_IS_STYLE (base_style), NULL);
 	g_return_val_if_fail (EEL_IS_BACKGROUND (background), NULL);
@@ -142,23 +169,65 @@ eel_background_style_new (GtkStyle *base_style,
 
 	GTK_STYLE_CLASS (g_type_class_peek (GTK_TYPE_STYLE))->copy (GTK_STYLE (style), base_style);
 
-	background_color = eel_background_get_color (background);
-	start_color_spec = eel_gradient_get_start_color_spec (background_color);
-	g_free (background_color);
-	eel_gdk_color_parse_with_white_default (start_color_spec, &color);
-	g_free (start_color_spec);
+	style->details->background = g_object_ref (background);
 	
-	GTK_STYLE (style)->bg[GTK_STATE_NORMAL] = color;
-	GTK_STYLE (style)->base[GTK_STATE_NORMAL] = color;
-	GTK_STYLE (style)->bg[GTK_STATE_ACTIVE] = color;
-	GTK_STYLE (style)->base[GTK_STATE_ACTIVE] = color;
-
 	return style;
 }
 
+static void
+eel_background_style_set_background (GtkStyle    *gtkstyle,
+				     GdkWindow   *window,
+				     GtkStateType state_type)
+{
+	EelBackgroundStyle *style;
+	EelBackground *background;
+	GdkColor color;
+	GdkPixmap *pixmap;
+	gboolean changes_with_size;
+	int window_width, window_height;
+	
+	style = EEL_BACKGROUND_STYLE (gtkstyle);
+
+	background = style->details->background;
+
+	g_return_if_fail (background != NULL);
+
+	gdk_drawable_get_size (window, &window_width, &window_height);
+	pixmap = eel_background_get_pixmap_and_color (background, window,
+						      window_width, window_height,
+						      &color,
+						      &changes_with_size);
+	gdk_rgb_find_color (gtkstyle->colormap, &color);
+
+	if (pixmap != NULL && !changes_with_size) {
+		gdk_window_set_back_pixmap (window, pixmap, FALSE);
+	} else {
+		gdk_window_set_background (window, &color);
+	}
+
+	if (pixmap != NULL) {
+		g_object_unref (pixmap);
+	}
+}
+
+static GtkStyle *
+eel_background_style_clone (GtkStyle *style)
+{
+	EelBackgroundStyle *new_style;
+
+	new_style = g_object_new (G_OBJECT_TYPE (style), NULL);
+	if (EEL_BACKGROUND_STYLE (style)->details->background != NULL) {
+		new_style->details->background = g_object_ref (EEL_BACKGROUND_STYLE (style)->details->background);
+	}
+	return GTK_STYLE (new_style);
+}
+
+
 static void
 eel_background_style_class_init (EelBackgroundStyleClass *class)
 {
 	G_OBJECT_CLASS (class)->finalize = eel_background_style_finalize;
+	GTK_STYLE_CLASS (class)->clone = eel_background_style_clone;
 	GTK_STYLE_CLASS (class)->draw_flat_box = eel_background_style_draw_flat_box;
+	GTK_STYLE_CLASS (class)->set_background = eel_background_style_set_background;
 }
