@@ -3,7 +3,6 @@
 #include <eel/eel-background.h>
 #include <libgnomevfs/gnome-vfs-init.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
-#include <gtk/gtkctree.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkwidget.h>
@@ -14,21 +13,23 @@
 int
 main  (int argc, char *argv[])
 {
-	GtkWidget *window, *ctree;
+	GtkWidget *window;
 	EelBackground *background;
 	char *image_uri;
+#if 0
 	char *titles[] = { "test 1", "test 2", "test 3"};
+#endif
 
 	gtk_init (&argc, &argv);
 
 	gnome_vfs_init ();
 
 	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-	ctree = gtk_ctree_new_with_titles (3, 2, titles);
 #if 0
+	ctree = gtk_ctree_new_with_titles (3, 2, titles);
 	gtk_container_add (GTK_CONTAINER (window), ctree);
 #endif
-	gtk_signal_connect (GTK_OBJECT (window), "destroy",
+	g_signal_connect (window, "destroy",
 			    gtk_main_quit, NULL);
 
 	background = eel_get_widget_background (window);
